@@ -35,7 +35,7 @@ import java.util.Scanner;
 public class pdftk {
 
   /* TODO: should read from compiler */
-  static final String PDFTK_VER = "3.0.9";
+  static final String PDFTK_VER = "3.1.1";
   static final boolean ASK_ABOUT_WARNINGS = false;
 
   static String prompt_for_password(String pass_name, String pass_app) {
@@ -254,7 +254,7 @@ public class pdftk {
             + "       * Generate FDF Data Stencils from PDF Forms\n"
             + "       * Apply a Background Watermark or a Foreground Stamp\n"
             + "       * Report PDF Metrics, Bookmarks and Metadata\n"
-            + "       * Add/Update PDF Bookmarks or Metadata\n"
+            + "       * Add/Update PDF Metrics, Bookmarks or Metadata\n"
             + "       * Attach Files to PDF Pages or the PDF Document\n"
             + "       * Unpack PDF Attachments\n"
             + "       * Burst a PDF Document into Single Pages\n"
@@ -524,10 +524,11 @@ public class pdftk {
             + "     entities. Does not create a new PDF.\n"
             + "\n"
             + "    update_info <info data filename | - | PROMPT>\n"
-            + "     Changes the bookmarks and metadata in a single PDF's Info\n"
-            + "     dictionary to match the input data file. The input data file\n"
-            + "     uses the same syntax as the output from dump_data. Non-ASCII\n"
-            + "     characters should be encoded as XML numerical entities.\n"
+            + "     Changes the bookmarks, page labels, page sizes, page rota‐\n"
+            + "     tions, and metadata in a single PDF's Info dictionary to\n"
+            + "     match the input data file. The input data file uses the same\n"
+            + "     syntax as the output from dump_data. Non-ASCII characters\n"
+            + "     should be encoded as XML numerical entities.\n"
             + "\n"
             + "     This operation does not change the metadata stored in the\n"
             + "     PDF's XMP stream, if it has one. (For this reason you should\n"
