@@ -25,13 +25,11 @@ package com.gitlab.pdftk_java;
 import java.io.IOException;
 import java.io.PrintStream;
 import java.nio.charset.StandardCharsets;
-import java.text.NumberFormat;
 import java.util.ArrayList;
 import java.util.Arrays;
 import java.util.HashSet;
 import java.util.Iterator;
 import java.util.List;
-import java.util.Locale;
 import java.util.Set;
 import org.apache.commons.lang3.StringEscapeUtils;
 import org.apache.commons.lang3.text.translate.CharSequenceTranslator;
@@ -52,10 +50,14 @@ class report {
   // Escape both Xml and Unicode entities
   // see
   // https://commons.apache.org/proper/commons-lang/javadocs/api-3.6/org/apache/commons/lang3/StringEscapeUtils.html#escapeXml-java.lang.String-
-  static CharSequenceTranslator XmlUnicodeEscaper =
-      StringEscapeUtils.ESCAPE_XML10.with(NumericEntityEscaper.between(0x7f, Integer.MAX_VALUE));
+  private static CharSequenceTranslator XmlUnicodeEscaper = null;
 
   static String OutputXmlString(String jss_p) {
+    if (XmlUnicodeEscaper == null) {
+      XmlUnicodeEscaper =
+          StringEscapeUtils.ESCAPE_XML10.with(
+              NumericEntityEscaper.between(0x7f, Integer.MAX_VALUE));
+    }
     return XmlUnicodeEscaper.translate(jss_p);
   }
 
@@ -75,11 +77,15 @@ class report {
     return "";
   }
 
-  static String OutputPdfName(PdfName pdfnn_p) {
+  static String OutputPdfName(PdfName pdfnn_p, boolean utf8_b) {
     if (pdfnn_p != null && pdfnn_p.isName()) {
-      String jnn_p = new String(pdfnn_p.getBytes());
+      String jnn_p = pdfnn_p.toString();
       jnn_p = PdfName.decodeName(jnn_p);
-      return OutputXmlString(jnn_p);
+      if (utf8_b) {
+        return OutputUtf8String(jnn_p);
+      } else {
+        return OutputXmlString(jnn_p);
+      }
     }
     return "";
   }
@@ -88,7 +94,7 @@ class report {
     if (pdfoo_p != null && pdfoo_p.isString()) {
       return OutputPdfString((PdfString) pdfoo_p, utf8_b);
     } else if (pdfoo_p != null && pdfoo_p.isName()) {
-      return OutputPdfName((PdfName) pdfoo_p);
+      return OutputPdfName((PdfName) pdfoo_p, utf8_b);
     }
     return null;
   }
@@ -162,13 +168,13 @@ class report {
       }
     }
 
-    void addApStates(PdfReader reader_p, PdfDictionary ap_p, PdfName state_name) {
+    void addApStates(PdfReader reader_p, PdfDictionary ap_p, PdfName state_name, boolean utf8_b) {
       if (ap_p.contains(state_name)) {
         PdfObject n_p = reader_p.getPdfObject(ap_p.get(state_name));
         if (n_p != null && n_p.isDictionary()) {
           Set<PdfName> n_set_p = ((PdfDictionary) n_p).getKeys();
           for (PdfName key_p : n_set_p) {
-            m_states.add(OutputPdfName(key_p));
+            m_states.add(OutputPdfName(key_p, utf8_b));
           }
         }
       }
@@ -365,9 +371,9 @@ class report {
 
               // this is one way to cull button option names: iterate over
               // appearance state names
-              acc_state.addApStates(reader_p, ap_p, PdfName.N);
-              acc_state.addApStates(reader_p, ap_p, PdfName.D);
-              acc_state.addApStates(reader_p, ap_p, PdfName.R);
+              acc_state.addApStates(reader_p, ap_p, PdfName.N, utf8_b);
+              acc_state.addApStates(reader_p, ap_p, PdfName.D, utf8_b);
+              acc_state.addApStates(reader_p, ap_p, PdfName.R, utf8_b);
             }
           }
 
@@ -501,7 +507,7 @@ class report {
     // subtype
     PdfObject subtype_p = reader_p.getPdfObject(annot_p.get(PdfName.SUBTYPE));
     if (subtype_p != null && subtype_p.isName()) {
-      ofs.println("AnnotSubtype: " + OutputPdfName((PdfName) subtype_p));
+      ofs.println("AnnotSubtype: " + OutputPdfName((PdfName) subtype_p, utf8_b));
     }
 
     ////
@@ -616,7 +622,7 @@ class report {
             PdfDictionary annot_p = (PdfDictionary) annot_po;
 
             PdfObject type_p = reader_p.getPdfObject(annot_p.get(PdfName.TYPE));
-            if (type_p.equals(PdfName.ANNOT)) {
+            if (type_p != null && type_p.equals(PdfName.ANNOT)) {
 
               PdfObject subtype_p = reader_p.getPdfObject(annot_p.get(PdfName.SUBTYPE));
 
@@ -645,20 +651,6 @@ class report {
     reader_p.resetReleasePage();
   }
 
-  //
-  static class PdfPageLabel {
-    static final String m_prefix = "PageLabel";
-    static final String m_begin_mark = "PageLabelBegin";
-    // TODO
-  };
-
-  //
-  class PdfPageMedia {
-    static final String m_prefix = "PageMedia";
-    static final String m_begin_mark = "PageMediaBegin";
-    // TODO
-  };
-
   static void ReportOutlines(
       PrintStream ofs, PdfDictionary outline_p, PdfReader reader_p, boolean utf8_b) {
     ArrayList<PdfBookmark> bookmark_data = new ArrayList<PdfBookmark>();
@@ -685,14 +677,12 @@ class report {
         if (0 < key_len
             && value_p.isString()
             && 0 < ((PdfString) value_p).toUnicodeString().length()) { // ouput
-          ofs.println(data_import.PdfInfo.m_begin_mark);
+          ofs.println(data_import.PdfInfo.BEGIN_MARK);
 
-          ofs.println(data_import.PdfInfo.m_key_label + " " + OutputPdfName(key_p));
+          ofs.println(data_import.PdfInfo.KEY_LABEL + " " + OutputPdfName(key_p, utf8_b));
 
           ofs.println(
-              data_import.PdfInfo.m_value_label
-                  + " "
-                  + OutputPdfString((PdfString) value_p, utf8_b));
+              data_import.PdfInfo.VALUE_LABEL + " " + OutputPdfString((PdfString) value_p, utf8_b));
         }
       }
 
@@ -721,53 +711,41 @@ class report {
 
         if (index_p != null && index_p.isNumber() && label_po != null && label_po.isDictionary()) {
           PdfDictionary label_p = (PdfDictionary) label_po;
-          ofs.println(PdfPageLabel.m_begin_mark);
+          PdfPageLabel pagelabel = new PdfPageLabel();
 
           // PageLabelNewIndex
-          ofs.println("PageLabelNewIndex: " + (long) (((PdfNumber) index_p).intValue()) + 1);
+          pagelabel.m_new_index = ((PdfNumber) index_p).intValue() + 1;
 
           { // PageLabelStart
-            ofs.print("PageLabelStart: ");
             PdfObject start_p = reader_p.getPdfObject(label_p.get(PdfName.ST));
             if (start_p != null && start_p.isNumber()) {
-              ofs.println((long) ((PdfNumber) start_p).intValue());
+              pagelabel.m_start = ((PdfNumber) start_p).intValue();
             } else {
-              ofs.println("1"); // the default
+              pagelabel.m_start = 1; // the default
             }
           }
 
           { // PageLabelPrefix
             PdfObject prefix_p = reader_p.getPdfObject(label_p.get(PdfName.P));
             if (prefix_p != null && prefix_p.isString()) {
-              ofs.println("PageLabelPrefix: " + OutputPdfString((PdfString) prefix_p, utf8_b));
+              pagelabel.m_prefix = OutputPdfString((PdfString) prefix_p, utf8_b);
             }
           }
 
           { // PageLabelNumStyle
-            PdfName r_p = new PdfName("r");
-            PdfName a_p = new PdfName("a");
-
             PdfObject style_p = reader_p.getPdfObject(label_p.get(PdfName.S));
-            ofs.print("PageLabelNumStyle: ");
             if (style_p != null && style_p.isName()) {
-              if (style_p.equals(PdfName.D)) {
-                ofs.println("DecimalArabicNumerals");
-              } else if (style_p.equals(PdfName.R)) {
-                ofs.println("UppercaseRomanNumerals");
-              } else if (style_p.equals(r_p)) {
-                ofs.println("LowercaseRomanNumerals");
-              } else if (style_p.equals(PdfName.A)) {
-                ofs.println("UppercaseLetters");
-              } else if (style_p.equals(a_p)) {
-                ofs.println("LowercaseLetters");
-              } else { // error
-                ofs.println("[PDFTK ERROR]");
-              }
+              pagelabel.m_num_style =
+                  PdfPageLabel.NumberingStyle.fromPdfName.getOrDefault(
+                          style_p, PdfPageLabel.NumberingStyle.ERROR)
+                      .representation;
             } else { // default
-              ofs.println("NoNumber");
+              pagelabel.m_num_style = PdfPageLabel.NumberingStyle.EMPTY.representation;
             }
           }
 
+          ofs.print(pagelabel);
+
         } else { // error
           ofs.println("[PDFTK ERROR: INVALID label_p IN ReportPageLabelNode]");
         }
@@ -794,6 +772,28 @@ class report {
     }
   }
 
+  static void ReportPageMedia(PrintStream ofs, int numPages, PdfReader reader_p, boolean utf8_b) {
+    // page metrics, rotation, stamptkData
+    for (int ii = 1; ii <= numPages; ++ii) {
+      PdfDictionary page_p = reader_p.getPageN(ii);
+
+      PdfPageMedia pagemedia = new PdfPageMedia();
+      pagemedia.m_number = ii;
+      pagemedia.m_rotation = reader_p.getPageRotation(page_p);
+      pagemedia.m_rect = reader_p.getPageSize(page_p);
+      pagemedia.m_crop = reader_p.getBoxSize(page_p, PdfName.CROPBOX);
+
+      ofs.print(pagemedia);
+
+      PdfString stamptkData_p = page_p.getAsString(PdfName.STAMPTKDATA);
+      if (stamptkData_p != null) {
+        ofs.println("PageMediaStamptkData: " + OutputPdfString(stamptkData_p, utf8_b));
+      }
+
+      reader_p.releasePage(ii);
+    }
+  }
+
   static void ReportOnPdf(PrintStream ofs, PdfReader reader_p, boolean utf8_b) {
     { // trailer data
       PdfDictionary trailer_p = reader_p.getTrailer();
@@ -890,60 +890,7 @@ class report {
       }
     }
 
-    { // page metrics, rotation, stamptkData
-      for (int ii = 1; ii <= numPages; ++ii) {
-        PdfDictionary page_p = reader_p.getPageN(ii);
-
-        ofs.println(PdfPageMedia.m_begin_mark);
-        ofs.println("PageMediaNumber: " + ii);
-
-        ofs.println("PageMediaRotation: " + reader_p.getPageRotation(page_p));
-
-        NumberFormat c_format = NumberFormat.getInstance(Locale.ROOT);
-
-        Rectangle page_rect_p = reader_p.getPageSize(page_p);
-        if (page_rect_p != null) {
-          ofs.println(
-              "PageMediaRect: "
-                  + c_format.format(page_rect_p.left())
-                  + " "
-                  + c_format.format(page_rect_p.bottom())
-                  + " "
-                  + c_format.format(page_rect_p.right())
-                  + " "
-                  + c_format.format(page_rect_p.top()));
-          ofs.println(
-              "PageMediaDimensions: "
-                  + c_format.format(page_rect_p.right() - page_rect_p.left())
-                  + " "
-                  + c_format.format(page_rect_p.top() - page_rect_p.bottom()));
-        }
-
-        Rectangle page_crop_p = reader_p.getBoxSize(page_p, PdfName.CROPBOX);
-        if (page_crop_p != null
-            && !(page_crop_p.left() == page_rect_p.left()
-                && page_crop_p.bottom() == page_rect_p.bottom()
-                && page_crop_p.right() == page_rect_p.right()
-                && page_crop_p.top() == page_rect_p.top())) {
-          ofs.println(
-              "PageMediaCropRect: "
-                  + c_format.format(page_crop_p.left())
-                  + " "
-                  + c_format.format(page_crop_p.bottom())
-                  + " "
-                  + c_format.format(page_crop_p.right())
-                  + " "
-                  + c_format.format(page_crop_p.top()));
-        }
-
-        PdfString stamptkData_p = page_p.getAsString(PdfName.STAMPTKDATA);
-        if (stamptkData_p != null) {
-          ofs.println("PageMediaStamptkData: " + OutputPdfString(stamptkData_p, utf8_b));
-        }
-
-        reader_p.releasePage(ii);
-      }
-    }
+    ReportPageMedia(ofs, numPages, reader_p, utf8_b);
 
     { // page labels (a/k/a logical page numbers)
       PdfDictionary catalog_p = reader_p.catalog;
